@@ -311,10 +311,17 @@ static int gettypecode(node t) {
      assert(istype(t));
      t = typeforward(t);
      if (t == null_T) return 0;
-     assert(t->body.type.name != NULL);
-     assert(t->body.type.name->body.symbol.name != NULL); /* to do: return an error message instead! */
-     assert(tostring(t) != NULL);
      if (t->body.type.runtime_type_code != -1) return t->body.type.runtime_type_code;
+     if(!t->body.type.name){
+         //unnamed type, warn about and give it an autogenerated name
+         //we don't have a position at this point, so there's no way to give the user a line number.
+         warning("Unnamed type requiring a typecode encountered, such a type is unlikely to be useful");
+         char temp_name[30];
+         snprintf(temp_name,sizeof(temp_name),"_%d_unnamed",numkeys+1);
+         t->body.type.name = newsymbol(UniqueString(temp_name),type__T,global_scope,defined_F);
+     }
+     assert(t->body.type.name->body.symbol.name != NULL);
+     assert(tostring(t) != NULL);
      if (db == NULL) opendb();
      datum key;
      key.dptr = tostring(t);
