@@ -108,10 +108,10 @@ public:
                      bool p_parens) const;
 
   bool is_homogeneous(const Poly& f) const;
-  void degree(const Poly& f, int *d) const;
+  void degree(const Poly& f, monomial d) const;
   // returns true if f is homogeneous, and sets already_allocated_degree_vector
   // to be the LCM of the exponent vectors of the degrees of all terms in f.
-  bool multi_degree(const Poly& f, int *already_allocated_degree_vector) const;
+  bool multi_degree(const Poly& f, monomial already_allocated_degree_vector) const;
 
   // Returns the pair (d, ishomog) where
   // d is the largest heft of the degree of each monomial in 'f'.
@@ -187,9 +187,7 @@ public:
 
   void debug_display(const Poly* f) const;
 
-  Poly* makeTerm(const ring_elem a, const int* monom) const;
-  // 'monom' is in 'varpower' format (i.e. from the front end)
-  // [2n+1 v1 e1 v2 e2 ... vn en], where each ei > 0, (in 'varpower' format)
+  Poly* makeTerm(const ring_elem a, const_varpower monom) const;
 
   void setZero(Poly& f) const // resets f to zero
   {
