@@ -3,7 +3,7 @@
 #ifndef _aring_QQ_flint_hpp_
 #define _aring_QQ_flint_hpp_
 
-#include <iosfwd>
+#include "interface/gmp-util.h"  // for mpz_reallocate_limbs
 
 // The following needs to be included before any flint files are included.
 #include <M2/gc-include.h>
@@ -29,7 +29,7 @@ namespace M2 {
    @brief wrapper for the flint fmpq_t integer representation
 */
 
-class ARingQQFlint : public RingInterface
+class ARingQQFlint : public SimpleARing<ARingQQFlint>
 {
  public:
   static const RingID ringID = ring_QQFlint;
@@ -90,7 +90,7 @@ class ARingQQFlint : public RingInterface
   }
 
   void init(ElementType& result) const { fmpq_init(&result); }
-  void clear(ElementType& result) const { fmpq_clear(&result); }
+  static void clear(ElementType& result) { fmpq_clear(&result); }
   void set(ElementType& result, const ElementType& a) const
   {
     fmpq_set(&result, &a);
@@ -234,6 +234,19 @@ class ARingQQFlint : public RingInterface
     fmpq_set_mpq(&result, a.get_mpq());
   }
 
+  /** @brief returns a read only view into the ring_elem
+   *  The return value of this function should not be modified,
+   *  since the contents point directly into the input ring_elem.
+   */
+  const ElementType from_ring_elem_const(const ring_elem& a) const
+  {
+    mpq_srcptr a1 = a.get_mpq();
+    fmpq result;
+    result.num = PTR_TO_COEFF(mpq_numref(a1));
+    result.den = PTR_TO_COEFF(mpq_denref(a1));
+    return result;
+  }
+
   /** @} */
 
   bool promote(const Ring* Rf, const ring_elem f, ElementType& result) const
