@@ -3,7 +3,9 @@
 #ifndef _aring_QQ_gmp_hpp_
 #define _aring_QQ_gmp_hpp_
 
-#include "interface/random.h"
+#include "interface/gmp-util.h"  // for mpz_reallocate_limbs
+#include "interface/random.h"    // for rawSetRandomQQ
+
 #include "aring.hpp"
 #include "buffer.hpp"
 #include "ringelem.hpp"
@@ -20,7 +22,7 @@ namespace M2 {
    @brief wrapper for the gmp mpq_t integer representation
 */
 
-class ARingQQGMP : public RingInterface
+class ARingQQGMP : public SimpleARing<ARingQQGMP>
 {
  public:
   static const RingID ringID = ring_QQ;
@@ -83,7 +85,7 @@ class ARingQQGMP : public RingInterface
   }
 
   void init(ElementType& result) const { mpq_init(&result); }
-  void clear(ElementType& result) const { mpq_clear(&result); }
+  static void clear(ElementType& result) { mpq_clear(&result); }
   void set(ElementType& result, const ElementType& a) const
   {
     mpq_set(&result, &a);
@@ -259,6 +261,11 @@ class ARingQQGMP : public RingInterface
     mpq_set(&result, a.get_mpq());
   }
 
+  const ElementType& from_ring_elem_const(const ring_elem& a) const
+  {
+    return *a.get_mpq();
+  }
+
 /** @} */
 
 #if 0    
