@@ -3,10 +3,11 @@
 #ifndef _aring_zz_flint_hpp_
 #define _aring_zz_flint_hpp_
 
+#include "interface/gmp-util.h"  // for mpz_reallocate_limbs
+
 #include "aring.hpp"
 #include "buffer.hpp"
 #include "ringelem.hpp"
-#include <iosfwd>
 #include "exceptions.hpp"
 #include "ZZ.hpp"
 
@@ -25,7 +26,7 @@ namespace M2 {
    @brief wrapper for the flint fmpz_t integer representation
 */
 
-class ARingZZ : public RingInterface
+class ARingZZ : public SimpleARing<ARingZZ>
 {
  public:
   static const RingID ringID = ring_ZZFlint;
@@ -81,7 +82,7 @@ class ARingZZ : public RingInterface
   }
 
   void init(ElementType& result) const { fmpz_init(&result); }
-  void clear(ElementType& result) const { fmpz_clear(&result); }
+  static void clear(ElementType& result) { fmpz_clear(&result); }
   void set(ElementType& result, const ElementType& a) const
   {
     fmpz_set(&result, &a);
@@ -232,6 +233,11 @@ void set_from_mpz(ElementType& result, mpz_srcptr a) const
     fmpz_set_mpz(&result, a.get_mpz());
   }
 
+  ElementType from_ring_elem_const(const ring_elem& a) const
+  {
+    return PTR_TO_COEFF(a.get_mpz());
+  }
+
   /** @} */
 
   bool promote(const Ring* Rf, const ring_elem f, ElementType& result) const
