@@ -3,9 +3,9 @@
 #ifndef _aring_zz_gmp_hpp_
 #define _aring_zz_gmp_hpp_
 
-#include <iosfwd>
+#include "interface/gmp-util.h"  // for mpz_reallocate_limbs
+#include "interface/random.h"    // for rawRandomInteger
 
-#include "interface/random.h"
 #include "aring.hpp"
 #include "buffer.hpp"
 #include "ringelem.hpp"
@@ -19,7 +19,7 @@ namespace M2 {
    @brief wrapper for the mpz_struct integer representation
 */
 
-class ARingZZGMP : public RingInterface
+class ARingZZGMP : public SimpleARing<ARingZZGMP>
 {
  public:
   static const RingID ringID = ring_ZZ;
@@ -75,7 +75,8 @@ class ARingZZGMP : public RingInterface
   }
 
   void init(ElementType& result) const { mpz_init(&result); }
-  void clear(ElementType& result) const { mpz_clear(&result); }
+  static void clear(ElementType& result) { mpz_clear(&result); }
+
   void set(ElementType& result, const ElementType& a) const
   {
     mpz_set(&result, &a);
@@ -228,6 +229,11 @@ class ARingZZGMP : public RingInterface
     mpz_set(&result, t);
   }
 
+  const ElementType& from_ring_elem_const(const ring_elem& a) const
+  {
+    return *a.get_mpz();
+  }
+
   /** @} */
 
   bool promote(const Ring* Rf, const ring_elem f, ElementType& result) const
