@@ -1,5 +1,4 @@
 #include "ringelem.hpp"
-#include "intarray.hpp"
 
 class Matrix;
 class RingElement;
@@ -40,6 +39,12 @@ extern "C" void dmonideal(MonomialIdeal *m);
 
 extern "C" void dintarray(M2_arrayint a);
 
+template <typename T>
+void dvector(gc_vector<T>& a);
+
+template <typename T>
+void dvector(std::vector<T>& a);
+
 extern "C" void dstash();
 
 extern "C" void dRRR(gmp_RR a);
