@@ -19,7 +19,8 @@ DetComputation::DetComputation(const Matrix *M0,
       col_set(NULL),
       this_row(0),
       this_col(0),
-      D(0)
+      D(0),
+      dynamic_cache(0)
 {
   if (do_exterior)
     {
@@ -90,6 +91,102 @@ DetComputation::~DetComputation()
     }
 }
 
+int DetComputation::make_dynamic_cache()
+{
+  // Traverse through matrix entries, find nonzero entries
+  int nonzero = -1;
+  for (int i = 0; i < M->n_rows(); ++i)
+    {
+      bool flag = false;
+      for (int j = 0; j < M->n_cols(); ++j)
+        {
+          if (!R->is_zero(M->elem(i, j)))
+            {
+              if (!flag)
+                {
+                  flag = true;
+                  dynamic_cache[0][++nonzero] = {};
+                  row_lookup[i] = nonzero;
+                }
+              dynamic_cache[0][nonzero][{{i}, {j}}] = M->elem(i, j);
+            }
+        }
+    }
+  int n_nonzero_rows = dynamic_cache[0].size();
+  for (int minor_size = 1; minor_size < p; ++minor_size)
+    {
+      for (int top_row = p - (minor_size + 1);
+           top_row <= n_nonzero_rows - minor_size;
+           ++top_row)
+        {
+          dynamic_cache[minor_size].insert({top_row, {}});
+          for (const auto &[pp, map] : dynamic_cache[minor_size - 1])
+            {
+              if (system_interrupted())
+                return COMP_INTERRUPTED;  // Allow interruption
+              if (pp <= top_row)
+                {
+                  continue;
+                }  // top_row wouldn't be the top row, so skip
+              for (auto x : dynamic_cache[0][top_row])
+                {
+                  for (const auto &[Didx, Dval] : map)
+                    {
+                      // Check if x and D live on distinct columns
+                      const std::vector<int> &Dcols = Didx.second;
+                      int xcol = x.first.second[0];
+                      auto col_find =
+                          std::find(Dcols.begin(), Dcols.end(), xcol);
+                      if (col_find != Dcols.end())
+                        {
+                          continue;
+                        }  // xcol found in Dcols, so skip
+                      // if no skip, compute a term in the cofactor
+                      ColRowIndices newKey = Didx;
+                      newKey.first.insert(newKey.first.begin(),
+                                          x.first.first[0]);
+                      // Get iterator to future location of xcol
+                      auto xcol_position = std::upper_bound(
+                          newKey.second.begin(), newKey.second.end(), xcol);
+                      newKey.second.insert(xcol_position, xcol);
+                      bool negate =
+                          ((xcol_position - newKey.second.begin()) % 2 != 0);
+                      // Insert, add or negate cofactor term
+                      auto search =
+                          dynamic_cache[minor_size][top_row].find(newKey);
+                      if (search == dynamic_cache[minor_size][top_row].end())
+                        {  // not found
+                          dynamic_cache[minor_size][top_row].insert(
+                              {newKey, R->mult(x.second, Dval)});
+                          if (negate)
+                            {
+                              R->negate_to(
+                                  dynamic_cache[minor_size][top_row][newKey]);
+                            }
+                        }
+                      else
+                        {  // found
+                          if (!negate)
+                            {
+                              R->add_to(
+                                  dynamic_cache[minor_size][top_row][newKey],
+                                  R->mult(x.second, Dval));
+                            }
+                          else
+                            {
+                              R->subtract_to(
+                                  dynamic_cache[minor_size][top_row][newKey],
+                                  R->mult(x.second, Dval));
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+  return COMP_DONE;
+}
+
 int DetComputation::step()
 // Compute one more determinant of size p.
 // increments I and/or J and updates 'dets', 'table'.
@@ -103,6 +200,25 @@ int DetComputation::step()
       get_minor(row_set, col_set, p, D);
       r = bareiss_det();
     }
+  else if (strategy == DET_DYNAMIC)
+    {
+      if (dynamic_cache.empty())
+        {  // Cache minors if needed
+          dynamic_cache.resize(p);
+          if (make_dynamic_cache() == COMP_INTERRUPTED) return COMP_INTERRUPTED;
+        }
+      std::vector<int> row_vec(p), col_vec(p);
+      for (int i = 0; i < p; ++i)
+        {
+          row_vec[i] = static_cast<int>(row_set[i]);
+          col_vec[i] = static_cast<int>(col_set[i]);
+        }
+      // Find row number
+      const Subdeterminant &map = dynamic_cache[p - 1][row_lookup[row_vec[0]]];
+      auto it = map.find({row_vec, col_vec});
+      if (it != map.end()) { r = it->second; }
+      else { r = ZERO_RINGELEM; }
+    }
   else
     r = calc_det(row_set, col_set, p);
 
@@ -314,7 +430,6 @@ Matrix /* or null */ *Matrix::exterior(int p, int strategy) const
           "determinant computations over RR or CC requires Strategy=>Cofactor");
       return 0;
     }
-
   DetComputation *d = new DetComputation(this, p, 1, strategy);
   d->calc(-1);
   Matrix *result = d->determinants();
@@ -343,7 +458,7 @@ Matrix /* or null */ *Matrix::minors(
     int n_to_compute,             // -1 means all
     M2_arrayintOrNull first_row,  // possibly NULL
     M2_arrayintOrNull first_col   // possibly NULL
-    ) const
+) const
 {
   if (strategy == DET_BAREISS && get_ring()->get_precision() > 0)
     {
