@@ -6,6 +6,7 @@
 #include <limits>
 #include <sstream>
 
+#include "Eschreyer.hpp"
 #include "hilb.hpp"
 #include "comp-gb.hpp"
 #include "comp-res.hpp"
@@ -70,6 +71,18 @@ const RingElement /* or null */ *IM2_Matrix_Hilbert(const Matrix *M)
   }
 }
 
+const Matrix *rawKernelOfGB(const Matrix *M)
+/* Assuming that the columns of G form a GB, this routine computes
+   a Groebner basis of the kernel of these elements, using an
+   appropriate Schreyer order on the source of G. */
+{
+  GBMatrix *N = new GBMatrix(M);
+  GBKernelComputation G(N);
+  G.calc();
+  GBMatrix *syz = G.get_syzygies();
+  return syz->to_matrix();
+}
+
 ///////////////////////////////////////////////////////////////////////////////////
 ///////// The following will be removed once the new code is functional
 /////////////
