@@ -3,6 +3,8 @@
 #ifndef _mat_elementary_ops_hpp_
 #define _mat_elementary_ops_hpp_
 
+#include <memory>
+
 template <typename MT>
 class MatElementaryOps;
 template <typename RT>
@@ -16,6 +18,7 @@ class MatElementaryOps<DMat<RT> >
  public:
   typedef DMat<RT> Mat;
   typedef typename Mat::ElementType ElementType;
+  typedef typename RT::Element Element;
   typedef typename Mat::Iterator Iterator;
   typedef typename Mat::ConstIterator ConstIterator;
 
@@ -142,8 +145,7 @@ class MatElementaryOps<DMat<RT> >
     assert(j < mat.numRows());
     assert(i != j);
 
-    ElementType f;
-    mat.ring().init(f);
+    Element f(mat.ring());
     mat.ring().set_zero(f);
 
     auto loc1 = mat.rowBegin(i);
@@ -155,7 +157,6 @@ class MatElementaryOps<DMat<RT> >
         mat.ring().mult(f, r, *loc2);
         mat.ring().add(*loc1, f, *loc1);
       }
-    mat.ring().clear(f);
   }
 
   static void column_op(Mat& mat, size_t i, const ElementType& r, size_t j)
@@ -165,8 +166,7 @@ class MatElementaryOps<DMat<RT> >
     assert(j < mat.numColumns());
     assert(i != j);
 
-    ElementType f;
-    mat.ring().init(f);
+    Element f(mat.ring());
     mat.ring().set_zero(f);
 
     auto loc1 = mat.columnBegin(i);
@@ -178,7 +178,6 @@ class MatElementaryOps<DMat<RT> >
         mat.ring().mult(f, r, *loc2);
         mat.ring().add(*loc1, f, *loc1);
       }
-    mat.ring().clear(f);
   }
 
  private:
@@ -191,11 +190,7 @@ class MatElementaryOps<DMat<RT> >
                      const ElementType& b1,
                      const ElementType& b2)
   {
-    ElementType f1, f2, g1, g2;
-    ring.init(f1);
-    ring.init(f2);
-    ring.init(g1);
-    ring.init(g2);
+    Element f1(ring), f2(ring), g1(ring), g2(ring);
     ring.set_zero(f1);
     ring.set_zero(f2);
     ring.set_zero(g1);
@@ -212,10 +207,6 @@ class MatElementaryOps<DMat<RT> >
         ring.set(*loc1, f1);
         ring.set(*loc2, g1);
       }
-    ring.clear(f1);
-    ring.clear(f2);
-    ring.clear(g1);
-    ring.clear(g2);
   }
 
  public:
@@ -275,8 +266,7 @@ class MatElementaryOps<DMat<RT> >
     auto loc2 = mat.columnBegin(j);
     auto end = mat.columnEnd(i);
 
-    ElementType f;
-    mat.ring().init(f);
+    Element f(mat.ring());
     mat.ring().set_zero(f);
     mat.ring().set_zero(result);
     for (; loc1 != end; ++loc1, ++loc2)
@@ -284,7 +274,6 @@ class MatElementaryOps<DMat<RT> >
         mat.ring().mult(f, *loc1, *loc2);
         mat.ring().add(result, result, f);
       }
-    mat.ring().clear(f);
   }
 
   static bool row_permute(Mat& mat, size_t start_row, M2_arrayint perm)
@@ -292,7 +281,7 @@ class MatElementaryOps<DMat<RT> >
     // We copy one row to another location for each cycle in 'perm' of length >
     // 1.
     size_t nrows_to_permute = perm->len;
-    bool* done = newarray_atomic(bool, nrows_to_permute);
+    std::unique_ptr<bool[]> done(new bool[nrows_to_permute]);
     for (size_t i = 0; i < nrows_to_permute; i++) done[i] = true;
     for (size_t i = 0; i < nrows_to_permute; i++)
       {
@@ -300,13 +289,11 @@ class MatElementaryOps<DMat<RT> >
         if (!done[j])
           {
             ERROR("expected permutation");
-            freemem(done);
             return false;
           }
         done[j] = false;
       }
-    ElementType* tmp = newarray_clear(ElementType, mat.numColumns());
-    for (size_t c = 0; c < mat.numColumns(); c++) mat.ring().init(tmp[c]);
+    typename RT::ElementArray tmp(mat.ring(), mat.numColumns());
     size_t next = 0;
 
     while (next < nrows_to_permute)
@@ -320,7 +307,7 @@ class MatElementaryOps<DMat<RT> >
             // store row 'next' into tmp
             auto loc1 = mat.rowBegin(start_row + next);
             auto end1 = mat.rowEnd(start_row + next);
-            copy_from_iter(mat.ring(), tmp, tmp + mat.numColumns(), loc1);
+            copy_from_iter(mat.ring(), tmp.data(), tmp.data()+mat.numColumns(), loc1);
 
             size_t r = next;
             for (;;)
@@ -337,13 +324,11 @@ class MatElementaryOps<DMat<RT> >
                 r = perm->array[r];
               }
             // Now copy tmp back
-            copy_from_iter(mat.ring(), loc1, end1, tmp);
+            // TODO this is even more evil than the above
+            copy_from_iter(mat.ring(), loc1, end1, tmp.data());
             done[r] = true;
           }
       }
-    for (size_t c = 0; c < mat.numColumns(); c++) mat.ring().clear(tmp[c]);
-    freemem(tmp);
-    freemem(done);
     return true;
   }
 
@@ -352,7 +337,7 @@ class MatElementaryOps<DMat<RT> >
     // We copy one column to another location for each cycle in 'perm' of length
     // > 1.
     size_t ncols_to_permute = perm->len;
-    bool* done = newarray_atomic(bool, ncols_to_permute);
+    std::unique_ptr<bool[]> done(new bool[ncols_to_permute]);
     for (size_t i = 0; i < ncols_to_permute; i++) done[i] = true;
     for (size_t i = 0; i < ncols_to_permute; i++)
       {
@@ -360,13 +345,11 @@ class MatElementaryOps<DMat<RT> >
         if (!done[j])
           {
             ERROR("expected permutation");
-            freemem(done);
             return false;
           }
         done[j] = false;
       }
-    ElementType* tmp = newarray_clear(ElementType, mat.numRows());
-    for (size_t r = 0; r < mat.numRows(); r++) mat.ring().init(tmp[r]);
+    typename RT::ElementArray tmp(mat.ring(), mat.numRows());
     size_t next = 0;
 
     while (next < ncols_to_permute)
@@ -380,7 +363,7 @@ class MatElementaryOps<DMat<RT> >
             auto loc1 = mat.columnBegin(start_col + next);
             auto end1 = mat.columnEnd(start_col + next);
             // store col 'next' into tmp
-            copy_from_iter(mat.ring(), tmp, tmp + mat.numRows(), loc1);
+            copy_from_iter(mat.ring(), tmp.data(), tmp.data() + mat.numRows(), loc1);
 
             size_t r = next;
             for (;;)
@@ -398,13 +381,10 @@ class MatElementaryOps<DMat<RT> >
                 r = perm->array[r];
               }
             // Now copy tmp back
-            copy_from_iter(mat.ring(), loc1, end1, tmp);
+            copy_from_iter(mat.ring(), loc1, end1, tmp.data());
             done[r] = true;
           }
       }
-    for (size_t r = 0; r < mat.numRows(); r++) mat.ring().clear(tmp[r]);
-    freemem(tmp);
-    freemem(done);
     return true;
   }
 
@@ -500,12 +480,8 @@ class MatElementaryOps<DMat<RT> >
     //   and in row nr.
     // Replace column nc with all zeros, except 1 in nr row.
 
-    typename Mat::ElementType pivot, coef, f, zero, one;
-    M.ring().init(pivot);
-    M.ring().init(coef);
-    M.ring().init(zero);
-    M.ring().init(one);
-    M.ring().init(f);
+    Element pivot(M.ring()), coef(M.ring()), f(M.ring()), zero(M.ring()),
+        one(M.ring());
     M.ring().set_from_long(zero, 0);
     M.ring().set_from_long(one, 1);
 
@@ -530,12 +506,6 @@ class MatElementaryOps<DMat<RT> >
 
     scale_column(M, nc, zero);
     setEntry(M, nr, nc, one);
-
-    M.ring().clear(pivot);
-    M.ring().clear(coef);
-    M.ring().clear(zero);
-    M.ring().clear(one);
-    M.ring().clear(f);
   }
 
  public:
@@ -545,9 +515,7 @@ class MatElementaryOps<DMat<RT> >
     size_t nr = M.numRows() - 1;
     size_t nc = M.numColumns() - 1;
 
-    typename Mat::ElementType one, minus_one;
-    M.ring().init(one);
-    M.ring().init(minus_one);
+    Element one(M.ring()), minus_one(M.ring());
     M.ring().set_from_long(one, 1);
     M.ring().set_from_long(minus_one, -1);
 
@@ -610,9 +578,6 @@ class MatElementaryOps<DMat<RT> >
             break;
           }
       }
-
-    M.ring().clear(minus_one);
-    M.ring().clear(one);
   }
   //////////////////////////////////
 
