@@ -1,22 +1,28 @@
 // Copyright 1995-2004 Michael E. Stillman
 
+#include "matrix.hpp"
+
+#include <algorithm>
+#include <iostream>
+#include <vector>
+
+#include "interface/gmp-util.h"  // for mpz_reallocate_limbs
 #include "interface/random.h"
+
+#include "ExponentVector.hpp"
 #include "style.hpp"
 #include "text-io.hpp"
 #include "ring.hpp"
-#include "matrix.hpp"
 #include "comb.hpp"
 #include "polyring.hpp"
 #include "assprime.hpp"
 #include "monideal.hpp"
 #include "relem.hpp"
 #include "freemod.hpp"
-#include "ntuple.hpp"
+#include "util.hpp"
 
 #include "exptable.h"
 
-#include <vector>
-#include <iostream>
 #include "matrix-con.hpp"
 
 #include "monomial-collection.hpp"
@@ -25,12 +31,12 @@
 
 Matrix::Matrix(const FreeModule *rows0,
                const FreeModule *cols0,
-               const int *degree_shift0,
+               const_monomial degree_shift0,
                VECTOR(vec) & entries0)
 {
   mTarget = const_cast<FreeModule *>(rows0);
   mSource = const_cast<FreeModule *>(cols0);
-  mDegreeShift = const_cast<int *>(degree_shift0);
+  mDegreeShift = const_cast<monomial>(degree_shift0);
   for (int i = 0; i < cols0->rank(); i++) mEntries.push_back(entries0[i]);
 }
 
@@ -114,7 +120,7 @@ const Matrix /* or null */ *Matrix::make(const FreeModule *target,
         }
     }
 
-  int *degshift = R->degree_monoid()->make_one();
+  monomial degshift = R->degree_monoid()->make_one();
   R->degree_monoid()->from_expvector(deg->array, degshift);
   MatrixConstructor mat(target, source, degshift);
 
@@ -208,7 +214,7 @@ const Matrix /* or null */ *Matrix::make_sparse(
 #warning "check that all rings are correct, give error otherwise"
 #endif
   const Ring *R = target->get_ring();
-  int *degshift = R->degree_monoid()->make_one();
+  monomial degshift = R->degree_monoid()->make_one();
   R->degree_monoid()->from_expvector(deg->array, degshift);
 
   MatrixConstructor mat(target, source, degshift);
@@ -241,7 +247,7 @@ const Matrix /* or null */ *Matrix::remake(const FreeModule *target,
       return nullptr;
     }
 
-  int *degshift = R->degree_monoid()->make_one();
+  monomial degshift = R->degree_monoid()->make_one();
   R->degree_monoid()->from_expvector(deg->array, degshift);
   MatrixConstructor mat(target, source, degshift);
   for (int i = 0; i < source->rank(); i++)
@@ -279,13 +285,13 @@ const Matrix /* or null */ *Matrix::make(const MonomialIdeal *mi)
       return nullptr;
     }
   const Monoid *M = P->getMonoid();
-  int *mon = M->make_one();
+  monomial mon = M->make_one();
 
-  MatrixConstructor mat(P->make_FreeModule(1), mi->length());
+  MatrixConstructor mat(P->make_FreeModule(1), mi->size());
   int next = 0;
-  for (Index<MonomialIdeal> i = mi->last(); i.valid(); i--)
+  for (auto i = mi->beginAtLast(); i != mi->end(); --i)  // TODO MES: should go from last() via --i to end()...
     {
-      M->from_varpower((*mi)[i]->monom().raw(), mon);
+      M->from_varpower(i->monom().data(), mon);
       ring_elem f =
           P->make_flat_term(P->getCoefficientRing()->from_long(1), mon);
       mat.set_entry(0, next++, f);
@@ -323,7 +329,7 @@ bool Matrix::is_zero() const
 bool Matrix::is_homogeneous() const
 {
   if (!get_ring()->is_graded()) return 0;
-  int *d = degree_monoid()->make_one();
+  monomial d = degree_monoid()->make_one();
   for (int i = 0; i < n_cols(); i++)
     {
       if (elem(i) == nullptr) continue;
@@ -333,7 +339,7 @@ bool Matrix::is_homogeneous() const
           return 0;
         }
 
-      get_ring()->vec_degree(rows(), elem(i), d);
+      get_ring()->vec_multi_degree(rows(), elem(i), d);
       degree_monoid()->divide(d, degree_shift(), d);
       if (0 != degree_monoid()->compare(d, cols()->degree(i)))
         {
@@ -390,7 +396,7 @@ Matrix *Matrix::operator+(const Matrix &m) const
   const Ring *R = get_ring();
   const FreeModule *F = rows();
   const FreeModule *G = cols();
-  const int *deg;
+  const_monomial deg;
 
   if (!rows()->is_equal(m.rows())) F = R->make_FreeModule(n_rows());
 
@@ -428,7 +434,7 @@ Matrix *Matrix::operator-(const Matrix &m) const
   const Ring *R = get_ring();
   const FreeModule *F = rows();
   const FreeModule *G = cols();
-  const int *deg;
+  const_monomial deg;
 
   if (!rows()->is_equal(m.rows())) F = R->make_FreeModule(n_rows());
 
@@ -459,20 +465,32 @@ Matrix /* or null */ *Matrix::sub_matrix(M2_arrayint r, M2_arrayint c) const
   const FreeModule *G = cols()->sub_space(c);
   if (F == NULL || G == NULL) return nullptr;
 
-  int *trans = newarray_atomic(int, n_rows());
-  for (int i = 0; i < n_rows(); i++) trans[i] = -1;
+  int *minrow = newarray_atomic(int, n_rows());
+  int *maxrow = newarray_atomic(int, n_rows());
+  for (int i = 0; i < n_rows(); i++)
+    {
+      minrow[i] = n_rows();
+      maxrow[i] = -1;
+    }
 
-  for (unsigned j = 0; j < r->len; j++)
-    if (r->array[j] >= 0 && r->array[j] < n_rows()) trans[r->array[j]] = j;
+  for (int i = 0; i < r->len; i++)
+    if (r->array[i] >= 0 && r->array[i] < n_rows())
+      {
+        minrow[r->array[i]] = std::min(minrow[r->array[i]], i);
+        maxrow[r->array[i]] = std::max(maxrow[r->array[i]], i);
+      }
 
   MatrixConstructor mat(F, G, degree_shift());
-  for (unsigned int i = 0; i < c->len; i++)
+  for (size_t j = 0; j < c->len; j++)
     {
-      vec v = elem(c->array[i]);
+      vec v = elem(c->array[j]);
       for (; v != NULL; v = v->next)
-        if (trans[v->comp] != -1) mat.set_entry(trans[v->comp], i, v->coeff);
+        for (int i = minrow[v->comp]; i <= maxrow[v->comp]; i++)
+          if (v->comp == r->array[i])
+            mat.set_entry(i, j, v->coeff);
     }
-  freemem(trans);
+  freemem(minrow);
+  freemem(maxrow);
   return mat.to_matrix();
 }
 
@@ -562,7 +580,7 @@ Matrix *Matrix::transpose() const
 Matrix *Matrix::scalar_mult(const ring_elem r, bool opposite_mult) const
 {
   const Ring *R = get_ring();
-  int *deg = degree_monoid()->make_one();
+  monomial deg = degree_monoid()->make_one();
   if (!R->is_zero(r)) R->degree(r, deg);
   degree_monoid()->mult(deg, degree_shift(), deg);
   MatrixConstructor mat(rows(), cols(), deg);
@@ -607,7 +625,7 @@ Matrix *Matrix::direct_sum(const Matrix *m) const
     }
 
   // direct_sum ignores the degree shift of each summand.
-  /// const int *deg;
+  /// const_monomial deg;
   ///  if (EQ == degree_monoid()->compare(degree_shift(), m->degree_shift()))
   ///    deg = degree_shift();
   ///  else
@@ -641,7 +659,7 @@ Matrix *Matrix::mult(const Matrix *m, bool opposite_mult) const
       return nullptr;
     }
 
-  int *deg = degree_monoid()->make_new(degree_shift());
+  monomial deg = degree_monoid()->make_new(degree_shift());
   degree_monoid()->mult(deg, m->degree_shift(), deg);
 
   MatrixConstructor mat(rows(), m->cols(), deg);
@@ -747,7 +765,7 @@ Matrix *Matrix::tensor(const Matrix *m) const
 
   const FreeModule *F = rows()->tensor(m->rows());
   const FreeModule *G = cols()->tensor(m->cols());
-  int *deg = degree_monoid()->make_new(degree_shift());
+  monomial deg = degree_monoid()->make_new(degree_shift());
   degree_monoid()->mult(deg, m->degree_shift(), deg);
 
   MatrixConstructor mat(F, G, deg);
@@ -777,7 +795,7 @@ Matrix *Matrix::diff(const Matrix *m, int use_coef) const
   const FreeModule *F = F1->tensor(m->rows());
   FreeModule *G1 = cols()->transpose();
   const FreeModule *G = G1->tensor(m->cols());
-  int *deg = degree_monoid()->make_one();
+  monomial deg = degree_monoid()->make_one();
   degree_monoid()->divide(m->degree_shift(), degree_shift(), deg);
   freemem(F1);
   freemem(G1);
@@ -861,9 +879,9 @@ Matrix *Matrix::lead_term(int nparts) const
 //       if (v == NULL) continue;
 //       // Reduce each one in turn, and replace.
 //       Bag *junk_bag;
-//       vp.shrink(0);
+//       vp.resize(0);
 //       rows()->lead_varpower(v, vp);
-//       if (!mis[v->comp]->search(vp.raw(),junk_bag))
+//       if (!mis[v->comp]->search(vp.data(),junk_bag))
 //      {
 //        Bag *b = new Bag(indices->array[i], vp);
 //        mis[v->comp]->insert(b);
@@ -881,14 +899,14 @@ M2_arrayintOrNull Matrix::support() const
   {
     int n = R->n_vars();
     int nsupp = 0;
-    int *exp = newarray_atomic(int, R->n_vars());
-    int *exp2 = newarray_atomic(int, R->n_vars());
+    exponents_t exp = newarray_atomic(int, R->n_vars());
+    exponents_t exp2 = newarray_atomic(int, R->n_vars());
     for (int i = 0; i < R->n_vars(); i++) exp[i] = exp2[i] = 0;
     for (int j = 0; j < n_cols(); j++)
       for (vec v = elem(j); v != nullptr; v = v->next)
-        for (const Nterm *f = v->coeff; f != nullptr; f = f->next)
+        for (Nterm& f : v->coeff)
         {
-          R->getMonoid()->to_expvector(f->monom, exp2);
+          R->getMonoid()->to_expvector(f.monom, exp2);
           for (int k = 0; k < n; k++)
             if (exp2[k] != 0 && exp[k] == 0)
             {
@@ -986,7 +1004,7 @@ Matrix *Matrix::top_coefficients(Matrix *&monoms) const
 
 M2_arrayintOrNull Matrix::elim_vars(int nparts) const
 {
-  intarray keep;
+  gc_vector<int> keep;
   const PolynomialRing *P = get_ring()->cast_to_PolynomialRing();
   if (P == nullptr)
     {
@@ -995,15 +1013,13 @@ M2_arrayintOrNull Matrix::elim_vars(int nparts) const
     }
   int nslots = P->getMonoid()->n_slots(nparts);
   for (int i = 0; i < n_cols(); i++)
-    if (P->vec_in_subring(nslots, elem(i))) keep.append(i);
-  M2_arrayint result = M2_makearrayint(keep.length());
-  for (unsigned int i = 0; i < result->len; i++) result->array[i] = keep[i];
-  return result;
+    if (P->vec_in_subring(nslots, elem(i))) keep.push_back(i);
+  return stdvector_to_M2_arrayint<int>(keep);
 }
 
 M2_arrayintOrNull Matrix::elim_keep(int nparts) const
 {
-  intarray keep;
+  gc_vector<int> keep;
   const PolynomialRing *P = get_ring()->cast_to_PolynomialRing();
   if (P == nullptr)
     {
@@ -1012,10 +1028,8 @@ M2_arrayintOrNull Matrix::elim_keep(int nparts) const
     }
   int nslots = P->getMonoid()->n_slots(nparts);
   for (int i = 0; i < n_cols(); i++)
-    if (!P->vec_in_subring(nslots, elem(i))) keep.append(i);
-  M2_arrayint result = M2_makearrayint(keep.length());
-  for (unsigned int i = 0; i < result->len; i++) result->array[i] = keep[i];
-  return result;
+    if (!P->vec_in_subring(nslots, elem(i))) keep.push_back(i);
+  return stdvector_to_M2_arrayint<int>(keep);
 }
 
 Matrix *Matrix::divide_by_var(int n, int maxd, int &maxdivided) const
@@ -1088,7 +1102,7 @@ static MonomialIdeal *makemonideal(const Matrix *A)
       return nullptr;
     }
   const Monoid *M = P->getMonoid();
-  queue<Bag *> new_elems;
+  VECTOR(Bag *) new_elems;
 
   for (int i = 0; i < A->n_cols(); i++)
     {
@@ -1096,13 +1110,13 @@ static MonomialIdeal *makemonideal(const Matrix *A)
       if (v == nullptr) continue;
       Bag *b = new Bag(i);
       M->to_varpower(P->lead_flat_monomial(v->coeff), b->monom());
-      new_elems.insert(b);
+      new_elems.push_back(b);
     }
 
   MonomialIdeal *result = new MonomialIdeal(P, new_elems);
   return result;
 }
-static int signdivide(int n, const int *a, const int *b, int *exp)
+static int signdivide(int n, const_exponents a, const_exponents b, exponents_t exp)
 {
   int sign = 0;
   int sum = 0;
@@ -1146,26 +1160,25 @@ Matrix /* or null */ *Matrix::koszul_monomials(int nskew,
   for (int j = 0; j < nskew; j++) skew_list[j] = j;
   SkewMultiplication skew(nvars, nskew, skew_list);
   int ncols = c->n_cols();
-  const int *a;  // a monomial
 
-  int *aexp = newarray_atomic(int, nvars);
-  int *bexp = newarray_atomic(int, nvars);
-  int *result_exp = newarray_atomic(int, nvars);
-  int *m = M->make_one();
+  exponents_t aexp = newarray_atomic(int, nvars);
+  exponents_t bexp = newarray_atomic(int, nvars);
+  exponents_t result_exp = newarray_atomic(int, nvars);
+  monomial m = M->make_one();
   VECTOR(Bag *) divisors;
   for (int i = 0; i < ncols; i++)
     {
       if (c->elem(i) == nullptr) continue;
-      a = P->lead_flat_monomial(c->elem(i)->coeff);
+      const_monomial a = P->lead_flat_monomial(c->elem(i)->coeff);
       M->to_expvector(a, aexp);
       divisors.clear();
       A->find_all_divisors(aexp, divisors);
       for (int j = 0; j < divisors.size(); j++)
         {
           int rownum = divisors[j]->basis_elem();
-          const int *b = P->lead_flat_monomial(r->elem(rownum)->coeff);
+          const_monomial b = P->lead_flat_monomial(r->elem(rownum)->coeff);
           M->to_expvector(b, bexp);
-          ntuple::divide(nvars, aexp, bexp, result_exp);
+          exponents::divide(nvars, aexp, bexp, result_exp);
           int sign = skew.mult_sign(result_exp, bexp);
           if (sign != 0)
             {
@@ -1198,10 +1211,10 @@ Matrix /* or null */ *Matrix::koszul(const Matrix *r, const Matrix *c)
   int nvars = M->n_vars();
   int nrows = r->n_cols();
   int ncols = c->n_cols();
-  const int *a, *b;  // monomials
-  int *aexp = newarray_atomic(int, nvars);
-  int *bexp = newarray_atomic(int, nvars);
-  int *result_exp = newarray_atomic(int, nvars);
+  const_monomial a, b;
+  exponents_t aexp = newarray_atomic(int, nvars);
+  exponents_t bexp = newarray_atomic(int, nvars);
+  exponents_t result_exp = newarray_atomic(int, nvars);
   for (int i = 0; i < ncols; i++)
     {
       if (c->elem(i) == nullptr) continue;
@@ -1215,7 +1228,7 @@ Matrix /* or null */ *Matrix::koszul(const Matrix *r, const Matrix *c)
           int sign = signdivide(nvars, aexp, bexp, result_exp);
           if (sign != 0)
             {
-              int *m = M->make_one();
+              monomial m = M->make_one();
               M->from_expvector(result_exp, m);
               ring_elem s = (sign > 0 ? K->one() : K->minus_one());
               ring_elem f = P->make_flat_term(s, m);
@@ -1694,8 +1707,8 @@ Matrix /* or null */ *Matrix::monomials(M2_arrayint vars) const
       }
 
   // Now collect all of the monomials
-  int *mon = M->make_one();
-  int *exp = newarray_atomic(int, M->n_vars());
+  monomial mon = M->make_one();
+  exponents_t exp = newarray_atomic(int, M->n_vars());
   ring_elem one = K->from_long(1);
   exponent_table *E =
       exponent_table_new(50000, vars->len + 1);  // the +1 is for the component
@@ -1705,10 +1718,10 @@ Matrix /* or null */ *Matrix::monomials(M2_arrayint vars) const
       vec v = elem(c);
       for (; v != nullptr; v = v->next)
         {
-          for (Nterm *t = v->coeff; t != nullptr; t = t->next)
+          for (Nterm& t : v->coeff)
             {
-              int *exp1 = newarray_atomic(int, vars->len + 1);
-              M->to_expvector(t->monom, exp);
+              exponents_t exp1 = newarray_atomic(int, vars->len + 1);
+              M->to_expvector(t.monom, exp);
               for (unsigned int i = 0; i < vars->len; i++)
                 exp1[i] = exp[vars->array[i]];
               exp1[vars->len] = v->comp;
@@ -1723,7 +1736,7 @@ Matrix /* or null */ *Matrix::monomials(M2_arrayint vars) const
   for (int i = 0; i < nvars; i++) exp[i] = 0;
   for (int i = 0; monoms[i] != 0; i += 2)
     {
-      const int *exp1 = reinterpret_cast<const int *>(monoms[i]);
+      const_exponents exp1 = reinterpret_cast<const_exponents>(monoms[i]);
       for (unsigned int j = 0; j < vars->len; j++)
         exp[vars->array[j]] = exp1[j];
       int x = exp1[vars->len];  // component
@@ -1775,25 +1788,25 @@ static vec coeffs_of_vec(exponent_table *E,
   const PolynomialRing *P = F->get_ring()->cast_to_PolynomialRing();
   if (P == nullptr) return nullptr;
   const Monoid *M = P->getMonoid();
-  int *mon = M->make_one();
+  monomial mon = M->make_one();
 
   // At this point, we know that we have a polynomial ring
   int nvars = M->n_vars();
-  int *exp = newarray_atomic(int, nvars);
-  int *scratch_exp = newarray_atomic(int, 1 + vars->len);
+  exponents_t exp = newarray_atomic(int, nvars);
+  exponents_t scratch_exp = newarray_atomic(int, 1 + vars->len);
 
   vec result = nullptr;
   for (vec g = f; g != nullptr; g = g->next)
     {
-      for (Nterm *h = g->coeff; h != nullptr; h = h->next)
+      for (Nterm& h : g->coeff)
         {
-          M->to_expvector(h->monom, exp);
+          M->to_expvector(h.monom, exp);
           get_part_of_expvector(vars, exp, g->comp, scratch_exp);
           int val = static_cast<int>(exponent_table_get(E, scratch_exp));
           if (val > 0)
             {
               M->from_expvector(exp, mon);
-              ring_elem t = P->make_flat_term(h->coeff, mon);
+              ring_elem t = P->make_flat_term(h.coeff, mon);
               vec v = P->make_vec(val - 1, t);
               v->next = result;
               result = v;
@@ -1851,7 +1864,7 @@ Matrix /* or null */ *Matrix::coeffs(M2_arrayint vars,
               return nullptr;
             }
           ring_elem f = v->coeff;
-          const int *m = P->lead_flat_monomial(f);
+          const_monomial m = P->lead_flat_monomial(f);
           P->getMonoid()->to_expvector(m, EXP);
 
           // grab only that part of the monomial we need
@@ -1893,7 +1906,7 @@ MonomialIdeal *Matrix::make_monideal(
   bool coeffsZZ = (P->coefficient_type() == Ring::COEFF_ZZ &&
                    use_only_monomials_with_unit_coeffs);
   const Monoid *M = P->getMonoid();
-  queue<Bag *> new_elems;
+  VECTOR(Bag *) new_elems;
   for (int i = 0; i < n_cols(); i++)
     {
       vec v = elem(i);
@@ -1904,7 +1917,7 @@ MonomialIdeal *Matrix::make_monideal(
         continue;
       Bag *b = new Bag(i);
       M->to_varpower(P->lead_flat_monomial(w->coeff), b->monom());
-      new_elems.insert(b);
+      new_elems.push_back(b);
     }
 
   // If the base ring is a quotient ring, include these lead monomials.
@@ -1914,7 +1927,7 @@ MonomialIdeal *Matrix::make_monideal(
       if (coeffsZZ && !globalZZ->is_unit(f->coeff)) continue;
       Bag *b = new Bag(-1);
       M->to_varpower(f->monom, b->monom());
-      new_elems.insert(b);
+      new_elems.push_back(b);
     }
 
   // If the base ring has skew commuting variables, include their squares
@@ -1925,7 +1938,7 @@ MonomialIdeal *Matrix::make_monideal(
           {
             Bag *b = new Bag(-1);
             varpower::var(i, 2, b->monom());
-            new_elems.insert(b);
+            new_elems.push_back(b);
           }
     }
 
