@@ -1,15 +1,28 @@
-// (c) 1994  Michael E. Stillman
+// Copyright 1994-2021  Michael E. Stillman
+
 #ifndef _monideal_hh_
 #define _monideal_hh_
 
-#include "queue.hpp"
-#include "index.hpp"
-#include "varpower.hpp"
+#include "ExponentVector.hpp"
+#include "ExponentList.hpp"
 #include "int-bag.hpp"
+#include "monoid.hpp"
 #include "ring.hpp"
 #include "polyring.hpp"
+#include "mem.hpp"
 
-class MinimalPrimes;
+#if 0
+SparseMonomial: pointer to an array of ints.  First is the length.  [len, v1, e1, v2, e2, ..., vr, er]
+  len == 2r+1,
+  in comm case: v1 < v2 < ...< vr;
+  SparseMonomial::Iterator: i.var, i.exponent, or *i = <i.var, i.exponent>
+    for (auto a : monom) // a is a std::pair.
+  allocation: std::vector<int>, preallocated Range, MemoryBlock.
+  AllocatedSparseMonomial(sizinginfo)
+  LCM(a, b, allocatedspace) --> returns a SparseMonomial.
+
+
+#endif
 
 class Nmi_node : public our_new_delete  // monomial ideal internal node ///
 {
@@ -17,7 +30,7 @@ class Nmi_node : public our_new_delete  // monomial ideal internal node ///
   friend class AssociatedPrimes;
   friend class MinimalPrimes;
 
- protected:
+private:
   int var;
   int exp;
   Nmi_node *left;
@@ -42,8 +55,8 @@ class Nmi_node : public our_new_delete  // monomial ideal internal node ///
 
   Nmi_node *&down() { return val.down; }
   Bag *&baggage() { return val.bag; }
-  intarray &monom() { return val.bag->monom(); }              // varpower
-  const intarray &monom() const { return val.bag->monom(); }  // varpower
+  gc_vector<int> &monom() { return val.bag->monom(); }              // varpower
+  const gc_vector<int> &monom() const { return val.bag->monom(); }  // varpower
   void insert_to_left(Nmi_node *q)
   {
     q->header = header;
@@ -67,8 +80,8 @@ class MonomialIdeal : public EngineObject
   friend class MinimalPrimes;
 
  private:
-  Nmi_node *new_mi_node(int v, int e, Nmi_node *d);
-  Nmi_node *new_mi_node(int v, int e, Bag *b);
+  Nmi_node *new_internal_mi_node(int v, int e, Nmi_node *d);
+  Nmi_node *new_leaf_mi_node(int v, int e, Bag *b);
   void delete_mi_node(Nmi_node *p);
 
   Nmi_node *first_node() const;
@@ -82,29 +95,31 @@ class MonomialIdeal : public EngineObject
   void do_tree(Nmi_node *p, int depth, int indent, int disp) const;
   int debug_check(Nmi_node *p, const Nmi_node *up) const;
 
+  bool isWellFormed() const; // throws errors, or returns true, currently.
  protected:
   virtual unsigned int computeHashValue() const;
 
  public:
   MonomialIdeal(const PolynomialRing *RR, stash *mi_stash = 0);
   virtual ~MonomialIdeal() { remove_MonomialIdeal(); }
-  int length_of() const { return count / 2; }
-  MonomialIdeal(const PolynomialRing *R,
-                queue<Bag *> &elems,
-                stash *mi_stash = 0);
-  MonomialIdeal(const PolynomialRing *R,
-                queue<Bag *> &elems,
-                queue<Bag *> &rejects,
-                stash *mi_stash = 0);
 
+  MonomialIdeal(const PolynomialRing *R0,
+                VECTOR(Bag *) &elems, // we now own these elements
+                VECTOR(Bag *) &rejects, // except for the ones we place into here
+                stash *mi_stash0 = nullptr);
+
+  MonomialIdeal(const PolynomialRing *R0,
+                VECTOR(Bag *) &elems, // we now own these elements, and will free those not needed
+                stash *mi_stash0 = nullptr);
+  
   MonomialIdeal *copy() const;
 
   void remove_MonomialIdeal();     // frees all of the internal things
-  const int *first_elem() const;   // returns varpower
-  const int *second_elem() const;  // returns varpower
+  const_varpower first_elem() const;
+  const_varpower second_elem() const;
 
   // Informational
-  int length() const { return count / 2; }
+  int size() const { return count / 2; }
   int topvar() const { return (mi == NULL ? -1 : mi->var); }
   void text_out(buffer &o) const;
 
@@ -122,7 +137,7 @@ class MonomialIdeal : public EngineObject
 
   int insert(Bag *b);
 
-  void insert_w_deletions(Bag *b, queue<Bag *> &deletions);
+  void insert_w_deletions(Bag *b, VECTOR(Bag *) &deletions);
   // Insert 'm', removing any monomials divisible by 'm', and
   // returning their baggage in a list of moninfo *'s.
 
@@ -130,21 +145,67 @@ class MonomialIdeal : public EngineObject
   // Deletion.  Remove the lexicographically largest element, placing
   // it into b.  Return 1 if an element is removed.
 
-  int search_expvector(const int *m, Bag *&b) const;
+  int search_expvector(const_exponents m, Bag *&b) const;
   // Search.  Return whether a monomial which divides 'm' is
   // found.  If so, return the baggage.  'm' is assumed to be an
   // exponent vector of length larger than the top variable occurring
   // in 'this'
-  int search(const int *m, Bag *&b) const;
+  int search(const_varpower m, Bag *&b) const;
   // Search.  Return whether a monomial which divides 'm' is
   // found.  If so, return the baggage.  'm' is a varpower monomial.
-  void find_all_divisors(const int *exp, VECTOR(Bag *)& b) const;
+  void find_all_divisors(const_exponents exp, VECTOR(Bag *)& b) const;
   // Search. Return a list of all elements which divide 'exp'.
 
-  Bag *operator[](Index<MonomialIdeal> i) const;
-  Index<MonomialIdeal> first() const;
-  Index<MonomialIdeal> last() const;
+  class Iterator {
+  private:
+    const MonomialIdeal& mMonomialTable;
+    Nmi_node* mPointer;
+  public:
+    using iterator_category = std::forward_iterator_tag;
+    using difference_type   = std::ptrdiff_t;
+    using value_type        = Bag;
+    using pointer           = Bag*;  // or also value_type*
+    using reference         = Bag&;  // or also value_type&
 
+    
+    Iterator(const MonomialIdeal& MI)
+      : mMonomialTable(MI),
+        mPointer(MI.first_node())
+    {
+    }
+
+    // Start at the end.
+    Iterator(const MonomialIdeal& MI, int)
+      : mMonomialTable(MI),
+        mPointer(nullptr)
+    {
+    }
+
+    // Internal use: used to start at the end.
+    Iterator(const MonomialIdeal& MI, Nmi_node* p)
+      : mMonomialTable(MI),
+        mPointer(p)
+    {
+    }
+    
+    reference operator*() const { return * mPointer->val.bag; }
+    pointer operator->() { return mPointer->val.bag; }
+
+    // Prefix increment, decrement
+    Iterator& operator++() { mPointer = mMonomialTable.next(mPointer); return *this; }
+    Iterator& operator--() { mPointer = mMonomialTable.prev(mPointer); return *this; }
+    
+    // Postfix increment, decrement
+    Iterator operator++(int) { Iterator tmp = *this; ++(*this); return tmp; }
+    Iterator operator--(int) { Iterator tmp = *this; --(*this); return tmp; }
+
+    friend bool operator== (const Iterator& a, const Iterator& b) { return a.mPointer == b.mPointer; };
+    friend bool operator!= (const Iterator& a, const Iterator& b) { return a.mPointer != b.mPointer; };
+  };
+
+  Iterator begin() const { return Iterator(*this); }
+  Iterator beginAtLast() const { return Iterator(*this, last_node()); }
+  Iterator end() const { return Iterator(*this, 1); }
   void *next(void *p) const;
   void *prev(void *p) const;
   int valid(void *p) const { return (p != NULL); }
@@ -153,11 +214,11 @@ class MonomialIdeal : public EngineObject
 
   bool is_equal(const MonomialIdeal &mi) const;
 
-  MonomialIdeal *intersect(const int *m) const;  // m is a varpower monomial
+  MonomialIdeal *intersect(const_varpower m) const;
   MonomialIdeal *intersect(const MonomialIdeal &J) const;
-  MonomialIdeal *quotient(const int *m) const;  // m is a varpower monomial
+  MonomialIdeal *quotient(const_varpower m) const;
   MonomialIdeal *quotient(const MonomialIdeal &J) const;
-  MonomialIdeal *erase(const int *m) const;  // m is a varpower monomial
+  MonomialIdeal *erase(const_varpower m) const;
   MonomialIdeal *sat(const MonomialIdeal &J) const;
 
   M2_arrayint lcm() const;
@@ -204,23 +265,8 @@ inline void MonomialIdeal::insert_minimal(Bag *b)
   count += 2;
 }
 
-inline Bag *MonomialIdeal::operator[](Index<MonomialIdeal> i) const
-{
-  Nmi_node *p = reinterpret_cast<Nmi_node *>(i.val());
-  return p->baggage();
-}
-
 inline Nmi_node *MonomialIdeal::first_node() const { return next(mi); }
 inline Nmi_node *MonomialIdeal::last_node() const { return prev(mi); }
-inline Index<MonomialIdeal> MonomialIdeal::first() const
-{
-  return Index<MonomialIdeal>(next(reinterpret_cast<void *>(mi)), this);
-}
-
-inline Index<MonomialIdeal> MonomialIdeal::last() const
-{
-  return Index<MonomialIdeal>(prev(reinterpret_cast<void *>(mi)), this);
-}
 
 #endif
 
