@@ -8,7 +8,7 @@
 /********************/
 
 #if 0
-// static bool exponents_equal(int nvars, exponents a, exponents b)
+// static bool exponents_equal(int nvars, exponents_t a, exponents_t b)
 // {
 //   for (int i=0; i<nvars; i++)
 //     if (a[i] != b[i]) return false;
@@ -16,7 +16,7 @@
 // }
 #endif
 
-static bool exponents_greater(int nvars, exponents a, exponents b)
+static bool exponents_greater(int nvars, exponents_t a, exponents_t b)
 {
   for (int i = 0; i < nvars; i++)
     {
@@ -26,7 +26,7 @@ static bool exponents_greater(int nvars, exponents a, exponents b)
   return false;
 }
 
-static void exponents_show(FILE *fil, exponents exp, int nvars)
+static void exponents_show(FILE *fil, exponents_t exp, int nvars)
 /* This is only for debugging */
 {
   fprintf(fil, "[");
@@ -34,7 +34,7 @@ static void exponents_show(FILE *fil, exponents exp, int nvars)
   fprintf(fil, "]");
 }
 
-static unsigned long monomial_mask(int nvars, exponents exp)
+static unsigned long monomial_mask(int nvars, exponents_t exp)
 {
   unsigned long result = 0;
   int i, j;
@@ -83,7 +83,7 @@ MonomialTableZZ::~MonomialTableZZ()
 // NEW FUNCTION, 5/21/09.  If not functional in a few days, remove it!
 int MonomialTableZZ::find_divisors(int max,
                                    mpz_t coeff,
-                                   exponents exp,
+                                   exponents_t exp,
                                    int comp,
                                    VECTOR(int) *result_term_divisors,
                                    VECTOR(int) *result_mon_divisors) const,
@@ -144,7 +144,7 @@ int MonomialTableZZ::find_divisors(int max,
 
 int MonomialTableZZ::find_term_divisors(int max,
                                         mpz_srcptr coeff,
-                                        exponents exp,
+                                        exponents_t exp,
                                         int comp,
                                         VECTOR(mon_term *) * result) const
 /* max: the max number of divisors to find.
@@ -190,7 +190,7 @@ int MonomialTableZZ::find_term_divisors(int max,
   return nmatches;
 }
 
-bool MonomialTableZZ::is_weak_member(mpz_srcptr c, exponents exp, int comp) const
+bool MonomialTableZZ::is_weak_member(mpz_srcptr c, exponents_t exp, int comp) const
 // Is [c,exp,comp] in the submodule generated by the terms in 'this'?
 // Maybe a gbvector should be returned?
 {
@@ -239,12 +239,12 @@ bool MonomialTableZZ::is_weak_member(mpz_srcptr c, exponents exp, int comp) cons
   return false;
 }
 
-bool MonomialTableZZ::is_strong_member(mpz_srcptr c, exponents exp, int comp) const
+bool MonomialTableZZ::is_strong_member(mpz_srcptr c, exponents_t exp, int comp) const
 {
   return (find_term_divisors(1, c, exp, comp, 0) > 0);
 }
 
-int MonomialTableZZ::find_smallest_coeff_divisor(exponents exp, int comp) const
+int MonomialTableZZ::find_smallest_coeff_divisor(exponents_t exp, int comp) const
 {
   assert(comp >= 1);
   if (comp >= static_cast<int>(_head.size())) return -1;
@@ -279,7 +279,7 @@ int MonomialTableZZ::find_smallest_coeff_divisor(exponents exp, int comp) const
 }
 
 int MonomialTableZZ::find_monomial_divisors(int max,
-                                            exponents exp,
+                                            exponents_t exp,
                                             int comp,
                                             VECTOR(mon_term *) * result) const
 {
@@ -323,7 +323,7 @@ int MonomialTableZZ::find_monomial_divisors(int max,
 }
 
 MonomialTableZZ::mon_term *MonomialTableZZ::find_exact(mpz_srcptr coeff,
-                                                       exponents exp,
+                                                       exponents_t exp,
                                                        int comp) const
 {
   if (comp >= static_cast<int>(_head.size())) return 0;
@@ -349,7 +349,7 @@ MonomialTableZZ::mon_term *MonomialTableZZ::find_exact(mpz_srcptr coeff,
 }
 
 MonomialTableZZ::mon_term *MonomialTableZZ::find_exact_monomial(
-    exponents exp,
+    exponents_t exp,
     int comp,
     int first_val) const
 {
@@ -397,7 +397,7 @@ void MonomialTableZZ::change_coefficient(mon_term *t,
   t->_val = new_id;
 }
 
-void MonomialTableZZ::insert(mpz_srcptr coeff, exponents exp, int comp, int id)
+void MonomialTableZZ::insert(mpz_srcptr coeff, exponents_t exp, int comp, int id)
 {
   /* Insert 'exp' into the monomial table.  These are kept sorted in ascending
      order
@@ -442,15 +442,15 @@ void MonomialTableZZ::insert(mpz_srcptr coeff, exponents exp, int comp, int id)
  * Minimalization ***********
  ****************************/
 
-struct montable_sorter_ZZ : public std::binary_function<int, int, bool>
+struct montable_sorter_ZZ
 {
   int nvars;
   const VECTOR(mpz_srcptr) & coeffs;
-  const VECTOR(exponents) & exps;
+  const VECTOR(exponents_t) & exps;
   const VECTOR(int) & comps;
   montable_sorter_ZZ(int nvars0,
                      const VECTOR(mpz_srcptr) & coeffs0,
-                     const VECTOR(exponents) & exps0,
+                     const VECTOR(exponents_t) & exps0,
                      const VECTOR(int) & comps0)
       : nvars(nvars0), coeffs(coeffs0), exps(exps0), comps(comps0)
   {
@@ -458,8 +458,8 @@ struct montable_sorter_ZZ : public std::binary_function<int, int, bool>
 
   bool operator()(int x, int y)
   {
-    exponents xx = exps[x];
-    exponents yy = exps[y];
+    exponents_t xx = exps[x];
+    exponents_t yy = exps[y];
     for (int i = 0; i < nvars; i++)
       if (xx[i] < yy[i])
         return true;
@@ -477,8 +477,8 @@ struct montable_sorter_ZZ : public std::binary_function<int, int, bool>
 #if 0
 //   bool operator()(int x, int y) {
 //     int result = 0; // -1 is false, 1 is true
-//     exponents xx = exps[x];
-//     exponents yy = exps[y];
+//     exponents_t xx = exps[x];
+//     exponents_t yy = exps[y];
 //     for (int i=0; i<nvars; i++)
 //       if (xx[i] < yy[i]) {result = 1; break;}
 //       else if (xx[i] > yy[i]) {result = -1; break;}
@@ -496,7 +496,7 @@ struct montable_sorter_ZZ : public std::binary_function<int, int, bool>
 
 void MonomialTableZZ::show_weak(FILE *fil,
                                 mpz_srcptr coeff,
-                                exponents exp,
+                                exponents_t exp,
                                 int comp,
                                 int val) const
 {
@@ -512,7 +512,7 @@ void MonomialTableZZ::show_weak(FILE *fil,
 
 void MonomialTableZZ::find_weak_generators(int nvars,
                                            const VECTOR(mpz_srcptr) & coeffs,
-                                           const VECTOR(exponents) & exps,
+                                           const VECTOR(exponents_t) & exps,
                                            const VECTOR(int) & comps,
                                            VECTOR(int) & result_positions,
                                            bool use_stable_sort)
@@ -600,7 +600,7 @@ void MonomialTableZZ::find_weak_generators(int nvars,
 
 void MonomialTableZZ::find_strong_generators(int nvars,
                                              const VECTOR(mpz_srcptr) & coeffs,
-                                             const VECTOR(exponents) & exps,
+                                             const VECTOR(exponents_t) & exps,
                                              const VECTOR(int) & comps,
                                              VECTOR(int) & result_positions)
 {
@@ -650,7 +650,7 @@ void MonomialTableZZ::show_mon_term(buffer &o, mon_term *t) const
 
 void MonomialTableZZ::show_mon_term(buffer &o,
                                     mpz_srcptr coeff,
-                                    exponents lead,
+                                    exponents_t lead,
                                     int comp) const
 {
   if (coeff != nullptr)
