@@ -183,7 +183,7 @@ RingElement /* or null */ *RingElement::lead_coeff(const Ring *coeffR) const
 }
 
 RingElement /* or null */ *RingElement::get_coeff(const Ring *coeffR,
-                                                  const Monomial *m) const
+                                                  const EngineMonomial *m) const
 {
   const PolynomialRing *P = R->cast_to_PolynomialRing();
   if (P == 0)
@@ -194,7 +194,7 @@ RingElement /* or null */ *RingElement::get_coeff(const Ring *coeffR,
   return new RingElement(coeffR, P->get_coeff(coeffR, get_value(), m->ints()));
 }
 
-Monomial *RingElement::lead_monom(int nvars) const
+EngineMonomial *RingElement::lead_monom(int nvars) const
 {
   if (is_zero())
     {
@@ -204,13 +204,13 @@ Monomial *RingElement::lead_monom(int nvars) const
   const PolynomialRing *P = R->cast_to_PolynomialRing();
   if (P != nullptr)
     {
-      intarray resultvp;
+      gc_vector<int> resultvp;
       Nterm *t = get_value();
 
-      int *exp = newarray_atomic(int, nvars);
+      exponents_t exp = newarray_atomic(int, nvars);
       P->lead_logical_exponents(nvars, t, exp);
-      varpower::from_ntuple(nvars, exp, resultvp);
-      return Monomial::make(resultvp.raw());
+      varpower::from_expvector(nvars, exp, resultvp);
+      return EngineMonomial::make(resultvp.data());
     }
   const M2FreeAlgebraOrQuotient* Q = dynamic_cast<const M2FreeAlgebraOrQuotient*>(R);
   if (Q != nullptr)
@@ -455,6 +455,7 @@ bool RingElement::getSmallIntegerCoefficients(
     std::vector<long> &result_coeffs) const
 {
   const PolynomialRing *R = get_ring()->cast_to_PolynomialRing();
+  const Ring *K = R->getCoefficientRing();
   if (R == 0 || R->n_vars() != 1)
     {
       throw exc::engine_error(
@@ -472,10 +473,9 @@ bool RingElement::getSmallIntegerCoefficients(
   result_coeffs.resize(deg + 1);
   for (int i = 0; i <= deg; i++) result_coeffs[i] = 0;
   int exp[1];
-  for (Nterm *t = get_value(); t != NULL; t = t->next)
+  for (Nterm& t : get_value())
     {
-      std::pair<bool, long> res =
-          R->getCoefficientRing()->coerceToLongInteger(t->coeff);
+      std::pair<bool, long> res = K->coerceToLongInteger(t.coeff);
       if (not res.first)
         {
           // At this point, the answer is meaningless
@@ -484,7 +484,7 @@ bool RingElement::getSmallIntegerCoefficients(
         }
       long coeff = res.second;
 
-      R->getMonoid()->to_expvector(t->monom, exp);
+      R->getMonoid()->to_expvector(t.monom, exp);
       assert(exp[0] >= 0);
       assert(exp[0] <= deg);
       result_coeffs[exp[0]] = coeff;
