@@ -1,7 +1,9 @@
 // Copyright 1996.  Michael E. Stillman
 
-#include "res-a0-poly.hpp"
 #include "res-a0.hpp"
+
+#include "ExponentVector.hpp"
+#include "res-a0-poly.hpp"
 #include "geobucket.hpp"
 #include "buffer.hpp"
 #include "text-io.hpp"
@@ -187,7 +189,7 @@ enum ComputationStatusCode res2_comp::do_pairs_by_level(int level)
           res2term *g = &head;
           for (f = p->syz; f != NULL; f = f->next)
             {
-              if (f->comp->mi->length() > 0)
+              if (f->comp->mi->size() > 0)
                 {
                   g->next = R->new_term(K->copy(f->coeff), f->monom, f->comp);
                   g = g->next;
@@ -633,7 +635,7 @@ res2_pair *res2_comp::new_base_res2_pair(int i)
   p->degree = (short unsigned int)(generator_matrix->rows()->primary_degree(i) -
                                    lodegree);
   p->compare_num = i;
-  int *m = M->make_one();
+  monomial m = M->make_one();
   p->syz = R->new_term(K->from_long(1), m, p);  // circular link...
   M->remove(m);
   p->mi = new MonomialIdeal(P, mi_stash);
@@ -714,7 +716,7 @@ int res2_comp::compare_res2_pairs(res2_pair *f, res2_pair *g) const
   // Lots of different orders appear here, controlled by the above
   // static variables.
   // if compare(f,g) returns -1, this says place g BEFORE f on the list.
-  exponents EXP1, EXP2, EXP3, EXP4;
+  exponents_t EXP1, EXP2, EXP3, EXP4;
   int cmp, df, dg;
 
   if (compare_use_degree)
@@ -935,13 +937,11 @@ void res2_comp::sort_reduction(res2_pair *&p)
   sort_res2_pairs(p);
 }
 
-int res2_comp::sort_value(res2_pair *p, const int *sort_order) const
+int res2_comp::sort_value(res2_pair *p, const std::vector<int> sort_order) const
 {
-  exponents REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
+  exponents_t REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
   M->to_expvector(p->syz->monom, REDUCE_exp);
-  int result = 0;
-  for (int i = 0; i < P->n_vars(); i++) result += REDUCE_exp[i] * sort_order[i];
-  return result;
+  return exponents::weight(P->n_vars(), REDUCE_exp, sort_order);
 }
 
 //////////////////////////////////////////////
@@ -952,10 +952,9 @@ void res2_comp::new_pairs(res2_pair *p)
 // Create and insert all of the pairs which will have lead term 'p'.
 // This also places 'p' into the appropriate monomial ideal
 {
-  Index<MonomialIdeal> j;
-  queue<Bag *> elems;
-  intarray vp;  // This is 'p'.
-  intarray thisvp;
+  gc_vector<Bag*> elems;
+  gc_vector<int> vp;  // This is 'p'.
+  gc_vector<int> thisvp;
 
   monomial PAIRS_mon = ALLOCATE_MONOMIAL(monom_size);
 
@@ -973,8 +972,8 @@ void res2_comp::new_pairs(res2_pair *p)
 
   if (P->is_skew_commutative())
     {
-      int *exp = newarray_atomic(int, M->n_vars());
-      varpower::to_ntuple(M->n_vars(), vp.raw(), exp);
+      exponents_t exp = newarray_atomic(int, M->n_vars());
+      varpower::to_expvector(M->n_vars(), vp.data(), exp);
 
       int nskew = P->n_skew_commutative_vars();
       for (int v = 0; v < nskew; v++)
@@ -982,10 +981,10 @@ void res2_comp::new_pairs(res2_pair *p)
           int w = P->skew_variable(v);
           if (exp[w] > 0)
             {
-              thisvp.shrink(0);
+              thisvp.resize(0);
               varpower::var(w, 1, thisvp);
               Bag *b = new Bag(static_cast<void *>(0), thisvp);
-              elems.insert(b);
+              elems.push_back(b);
             }
         }
       freemem(exp);
@@ -996,17 +995,17 @@ void res2_comp::new_pairs(res2_pair *p)
   if (P->is_quotient_ring())
     {
       const MonomialIdeal *Rideal = P->get_quotient_monomials();
-      for (j = Rideal->first(); j.valid(); j++)
+      for (Bag& a : *Rideal)
         {
           // Compute (P->quotient_ideal->monom : p->monom)
           // and place this into a varpower and Bag, placing
           // that into 'elems'
-          thisvp.shrink(0);
-          varpower::quotient((*Rideal)[j]->monom().raw(), vp.raw(), thisvp);
-          if (varpower::is_equal((*Rideal)[j]->monom().raw(), thisvp.raw()))
+          thisvp.resize(0);
+          varpower::quotient(a.monom().data(), vp.data(), thisvp);
+          if (varpower::is_equal(a.monom().data(), thisvp.data()))
             continue;
           Bag *b = new Bag(static_cast<void *>(0), thisvp);
-          elems.insert(b);
+          elems.push_back(b);
         }
     }
 
@@ -1014,11 +1013,11 @@ void res2_comp::new_pairs(res2_pair *p)
   // The baggage of each of these is their corresponding res2_pair
 
   MonomialIdeal *mi_orig = p->syz->comp->mi;
-  for (j = mi_orig->first(); j.valid(); j++)
+  for (Bag& a : *mi_orig)
     {
-      Bag *b = new Bag((*mi_orig)[j]->basis_ptr());
-      varpower::quotient((*mi_orig)[j]->monom().raw(), vp.raw(), b->monom());
-      elems.insert(b);
+      Bag *b = new Bag(a.basis_ptr());
+      varpower::quotient(a.monom().data(), vp.data(), b->monom());
+      elems.push_back(b);
     }
 
   // Make this monomial ideal, and then run through each minimal generator
@@ -1027,18 +1026,15 @@ void res2_comp::new_pairs(res2_pair *p)
 
   mi_orig->insert_minimal(new Bag(p, vp));
 
-  queue<Bag *> rejects;
-  Bag *b;
-  MonomialIdeal mi(P, elems, rejects);
-  while (rejects.remove(b)) delete b;
+  MonomialIdeal mi(P, elems);
 
   if (M2_gbTrace >= 11) mi.debug_out(1);
 
-  int *m = M->make_one();
-  for (j = mi.first(); j.valid(); j++)
+  monomial m = M->make_one();
+  for (Bag& a : mi)
     {
-      res2_pair *second = reinterpret_cast<res2_pair *>(mi[j]->basis_ptr());
-      M->from_varpower(mi[j]->monom().raw(), m);
+      res2_pair *second = reinterpret_cast<res2_pair *>(a.basis_ptr());
+      M->from_varpower(a.monom().data(), m);
       M->mult(m, p->syz->monom, m);
 
       res2_pair *q = new_res2_pair(p, second, m);
@@ -1073,15 +1069,15 @@ int res2_comp::find_divisor(const MonomialIdeal *mi,
   result = reinterpret_cast<res2_pair *>((bb[0]->basis_ptr()));
   // Now search through, and find the best one.  If only one, just return it.
   if (M2_gbTrace >= 5)
-    if (mi->length() > 1)
+    if (mi->size() > 1)
       {
         buffer o;
-        o << ":" << mi->length() << "." << bb.size() << ":";
+        o << ":" << mi->size() << "." << bb.size() << ":";
         emit(o.str());
       }
   if (bb.size() == 1)
     {
-      if (mi->length() == 1)
+      if (mi->size() == 1)
         n_ones++;
       else
         n_unique++;
@@ -1114,7 +1110,7 @@ int res2_comp::find_divisor(const MonomialIdeal *mi,
 res2term *res2_comp::s_pair(res2term *f) const
 {
   res2term *result = NULL;
-  int *si = M->make_one();
+  monomial si = M->make_one();
   while (f != NULL)
     {
       M->divide(f->monom, f->comp->syz->monom, si);
@@ -1137,7 +1133,7 @@ res2_pair *res2_comp::reduce(res2term *&f,
 // place a pointer to the corresponding term in "pivot".
 {
   // 'lastterm' is used to append the next monomial to fsyz->syz
-  exponents REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
+  exponents_t REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
   monomial REDUCE_mon = ALLOCATE_MONOMIAL(monom_size);
 
   res2term *lastterm = (fsyz->next == NULL ? fsyz : fsyz->next);
@@ -1212,7 +1208,7 @@ res2_pair *res2_comp::reduce2(res2term *&f,
 // 'p' is just here for auto-reduction...
 {
   // 'lastterm' is used to append the next monomial to fsyz->syz
-  exponents REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
+  exponents_t REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
   monomial REDUCE_mon = ALLOCATE_MONOMIAL(monom_size);
 
   res2term *lastterm = (fsyz->next == NULL ? fsyz : fsyz->next);
@@ -1319,7 +1315,7 @@ res2_pair *res2_comp::reduce3(res2term *&f,
 // place a pointer to the corresponding term in "pivot".
 {
   // 'lastterm' is used to append the next monomial to fsyz->syz
-  exponents REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
+  exponents_t REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
   monomial REDUCE_mon = ALLOCATE_MONOMIAL(monom_size);
 
   res2term *lastterm = (fsyz->next == NULL ? fsyz : fsyz->next);
@@ -1414,7 +1410,7 @@ res2_pair *res2_comp::reduce4(res2term *&f,
 // 'p' is just here for auto-reduction...
 {
   // 'lastterm' is used to append the next monomial to fsyz->syz
-  exponents REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
+  exponents_t REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
   monomial REDUCE_mon = ALLOCATE_MONOMIAL(monom_size);
 
   res2term *lastterm = fsyz;
@@ -1505,7 +1501,7 @@ res2_pair *res2_comp::reduce_by_level(res2term *&f, res2term *&fsyz)
 // place a pointer to the corresponding term in "pivot".
 {
   // 'lastterm' is used to append the next monomial to fsyz->syz
-  exponents REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
+  exponents_t REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
   monomial REDUCE_mon = ALLOCATE_MONOMIAL(monom_size);
 
   res2term *lastterm = (fsyz->next == NULL ? fsyz : fsyz->next);
@@ -1558,7 +1554,7 @@ res2_pair *res2_comp::reduce_by_level(res2term *&f, res2term *&fsyz)
 res2_pair *res2_comp::reduce_heap_by_level(res2term *&f, res2term *&fsyz)
 {
   // 'lastterm' is used to append the next monomial to fsyz->syz
-  exponents REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
+  exponents_t REDUCE_exp = ALLOCATE_EXPONENTS(exp_size);
   monomial REDUCE_mon = ALLOCATE_MONOMIAL(monom_size);
 
   res2term *lastterm = (fsyz->next == NULL ? fsyz : fsyz->next);
@@ -1971,7 +1967,7 @@ void res2_comp::text_out(buffer &o, const res2_pair *p) const
 #if 0
 //   if (p->mi_exists)
 #endif
-  o << "[mi: " << p->mi->length() << "]";
+  o << "[mi: " << p->mi->size() << "]";
 #if 0
 //   else
 //     {
@@ -2041,7 +2037,7 @@ FreeModule *res2_comp::free_of(int i) const
   result = P->make_Schreyer_FreeModule();
   if (i < 0 || i >= resn.size()) return result;
 
-  int *deg = degree_monoid()->make_one();
+  monomial deg = degree_monoid()->make_one();
   int n = 0;
   for (res2_pair *p = resn[i]->pairs; p != NULL; p = p->next)
     {
@@ -2059,7 +2055,7 @@ FreeModule *res2_comp::minimal_free_of(int i) const
   if (i < 0 || i >= resn.size() - 1) return result;
   if (do_by_level > 0) return free_of(i);
 
-  int *deg = degree_monoid()->make_one();
+  monomial deg = degree_monoid()->make_one();
   int n = 0;
   for (res2_pair *p = resn[i]->pairs; p != NULL; p = p->next)
     if (p->syz_type == SYZ2_MINIMAL)
