@@ -128,7 +128,7 @@ class gb2_comp : public gb_node
 
   s_pair_heap *spairs;
   s_pair *these_pairs;
-  intarray total_pairs;
+  gc_vector<int> total_pairs;
 
   VECTOR(gb_elem *) gb;
   VECTOR(monideal_pair *) monideals;  // baggage for each is 'gb_elem *'
