@@ -638,10 +638,11 @@ ring_elem SchurRing2::mult(const ring_elem f, const ring_elem g) const
     }
 }
 
-void toVarpower(const_schur_partition a, intarray &result)
+void to_varpower(const_schur_partition a, gc_vector<int>& result)
 {
   int len = a[0];
-  int *result_vp = result.alloc(2 * len);
+  result.resize(2 * len);
+  int *result_vp = result.data();
   int *orig_result_vp = result_vp;
   result_vp++;
 
@@ -668,7 +669,7 @@ void toVarpower(const_schur_partition a, intarray &result)
 
   int newlen = static_cast<int>(result_vp - orig_result_vp);
   *orig_result_vp = newlen;
-  result.shrink(newlen);
+  result.resize(newlen);
 }
 
 engine_RawArrayPairOrNull SchurRing2::list_form(const Ring *coeffR,
@@ -694,15 +695,15 @@ engine_RawArrayPairOrNull SchurRing2::list_form(const Ring *coeffR,
   result->coeffs = coeffs;
 
   // Loop through the terms
-  intarray vp;
+  gc_vector<int> vp;
   schur_poly::iterator i = f1->begin();
   for (int next = 0; next < n; ++i, ++next)
     {
       coeffs->array[next] =
           RingElement::make_raw(coefficientRing, i.getCoefficient());
-      toVarpower(i.getMonomial(), vp);
-      monoms->array[next] = Monomial::make(vp.raw());
-      vp.shrink(0);
+      to_varpower(i.getMonomial(), vp);
+      monoms->array[next] = EngineMonomial::make(vp.data());
+      vp.resize(0);
     }
   return result;
 }
